@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2013-2020, The PurpleI2P Project
+* Copyright (c) 2013-2022, The PurpleI2P Project
 *
 * This file is part of Purple i2pd project and licensed under BSD3
 *
@@ -27,8 +27,8 @@ namespace client
 	const size_t I2CP_SESSION_BUFFER_SIZE = 4096;
 	const size_t I2CP_MAX_MESSAGE_LENGTH = 65535;
 	const size_t I2CP_MAX_SEND_QUEUE_SIZE = 1024*1024; // in bytes, 1M
-	const int I2CP_LEASESET_CREATION_TIMEOUT = 10; // in seconds	
-	
+	const int I2CP_LEASESET_CREATION_TIMEOUT = 10; // in seconds
+
 	const size_t I2CP_HEADER_LENGTH_OFFSET = 0;
 	const size_t I2CP_HEADER_TYPE_OFFSET = I2CP_HEADER_LENGTH_OFFSET + 4;
 	const size_t I2CP_HEADER_SIZE = I2CP_HEADER_TYPE_OFFSET + 1;
@@ -61,8 +61,16 @@ namespace client
 		eI2CPMessageStatusNoLeaseSet = 21
 	};
 
+	enum I2CPSessionStatus
+	{
+		eI2CPSessionStatusDestroyed = 0,
+		eI2CPSessionStatusCreated = 1,
+		eI2CPSessionStatusUpdated = 2,
+		eI2CPSessionStatusInvalid = 3,
+		eI2CPSessionStatusRefused = 4
+	};
+	
 	// params
-	const char I2CP_PARAM_DONT_PUBLISH_LEASESET[] = "i2cp.dontPublishLeaseSet";
 	const char I2CP_PARAM_MESSAGE_RELIABILITY[] = "i2cp.messageReliability";
 
 	class I2CPSession;
@@ -70,12 +78,12 @@ namespace client
 	{
 		public:
 
-			I2CPDestination (boost::asio::io_service& service, std::shared_ptr<I2CPSession> owner, 
+			I2CPDestination (boost::asio::io_service& service, std::shared_ptr<I2CPSession> owner,
 				std::shared_ptr<const i2p::data::IdentityEx> identity, bool isPublic, const std::map<std::string, std::string>& params);
 			~I2CPDestination () {};
 
 			void Stop ();
-			
+
 			void SetEncryptionPrivateKey (const uint8_t * key);
 			void SetEncryptionType (i2p::data::CryptoKeyType keyType) { m_EncryptionKeyType = keyType; };
 			void SetECIESx25519EncryptionPrivateKey (const uint8_t * key);
@@ -84,8 +92,8 @@ namespace client
 			void SendMsgTo (const uint8_t * payload, size_t len, const i2p::data::IdentHash& ident, uint32_t nonce); // called from I2CPSession
 
 			// implements LocalDestination
-			bool Decrypt (const uint8_t * encrypted, uint8_t * data, BN_CTX * ctx, i2p::data::CryptoKeyType preferredCrypto) const;
-			bool SupportsEncryptionType (i2p::data::CryptoKeyType keyType) const;			
+			bool Decrypt (const uint8_t * encrypted, uint8_t * data, i2p::data::CryptoKeyType preferredCrypto) const;
+			bool SupportsEncryptionType (i2p::data::CryptoKeyType keyType) const;
 			const uint8_t * GetEncryptionPublicKey (i2p::data::CryptoKeyType keyType) const; // for 4 only
 			std::shared_ptr<const i2p::data::IdentityEx> GetIdentity () const { return m_Identity; };
 
@@ -102,7 +110,7 @@ namespace client
 			bool SendMsg (std::shared_ptr<I2NPMessage> msg, std::shared_ptr<const i2p::data::LeaseSet> remote);
 
 			void PostCreateNewLeaseSet (std::vector<std::shared_ptr<i2p::tunnel::InboundTunnel> > tunnels);
-			
+
 		private:
 
 			std::shared_ptr<I2CPSession> m_Owner;
@@ -114,32 +122,27 @@ namespace client
 			uint64_t m_LeaseSetExpirationTime;
 			bool m_IsCreatingLeaseSet;
 			boost::asio::deadline_timer m_LeaseSetCreationTimer;
+			i2p::util::MemoryPoolMt<I2NPMessageBuffer<I2NP_MAX_MESSAGE_SIZE> > m_I2NPMsgsPool;
 	};
 
-	class RunnableI2CPDestination: private i2p::util::RunnableService, public I2CPDestination 
+	class RunnableI2CPDestination: private i2p::util::RunnableService, public I2CPDestination
 	{
 		public:
 
-			RunnableI2CPDestination (std::shared_ptr<I2CPSession> owner, std::shared_ptr<const i2p::data::IdentityEx> identity, 
-				bool isPublic, const std::map<std::string, std::string>& params);	
+			RunnableI2CPDestination (std::shared_ptr<I2CPSession> owner, std::shared_ptr<const i2p::data::IdentityEx> identity,
+				bool isPublic, const std::map<std::string, std::string>& params);
 			~RunnableI2CPDestination ();
 
 			void Start ();
 			void Stop ();
-	};	
-	
+	};
+
 	class I2CPServer;
 	class I2CPSession: public std::enable_shared_from_this<I2CPSession>
 	{
 		public:
 
-#ifdef ANDROID
-			typedef boost::asio::local::stream_protocol proto;
-#else
-			typedef boost::asio::ip::tcp proto;
-#endif
-
-			I2CPSession (I2CPServer& owner, std::shared_ptr<proto::socket> socket);
+			I2CPSession (I2CPServer& owner, std::shared_ptr<boost::asio::ip::tcp::socket> socket);
 
 			~I2CPSession ();
 
@@ -175,19 +178,19 @@ namespace client
 			void HandleReceivedPayload (const boost::system::error_code& ecode, std::size_t bytes_transferred);
 			void HandleMessage ();
 			void Terminate ();
-			
+
 			void HandleI2CPMessageSent (const boost::system::error_code& ecode, std::size_t bytes_transferred);
-			
+
 			std::string ExtractString (const uint8_t * buf, size_t len);
 			size_t PutString (uint8_t * buf, size_t len, const std::string& str);
 			void ExtractMapping (const uint8_t * buf, size_t len, std::map<std::string, std::string>& mapping);
-			void SendSessionStatusMessage (uint8_t status);
+			void SendSessionStatusMessage (I2CPSessionStatus status);
 			void SendHostReplyMessage (uint32_t requestID, std::shared_ptr<const i2p::data::IdentityEx> identity);
 
 		private:
 
 			I2CPServer& m_Owner;
-			std::shared_ptr<proto::socket> m_Socket;
+			std::shared_ptr<boost::asio::ip::tcp::socket> m_Socket;
 			uint8_t m_Header[I2CP_HEADER_SIZE], m_Payload[I2CP_MAX_MESSAGE_LENGTH];
 			size_t m_PayloadLen;
 
@@ -199,7 +202,7 @@ namespace client
 			// to client
 			bool m_IsSending;
 			uint8_t m_SendBuffer[I2CP_MAX_MESSAGE_LENGTH];
-			i2p::stream::SendBufferQueue m_SendQueue; 
+			i2p::stream::SendBufferQueue m_SendQueue;
 	};
 	typedef void (I2CPSession::*I2CPMessageHandler)(const uint8_t * buf, size_t len);
 
@@ -214,9 +217,10 @@ namespace client
 			void Stop ();
 			boost::asio::io_service& GetService () { return GetIOService (); };
 			bool IsSingleThread () const { return m_IsSingleThread; };
-			
+
 			bool InsertSession (std::shared_ptr<I2CPSession> session);
 			void RemoveSession (uint16_t sessionID);
+			std::shared_ptr<I2CPSession> FindSessionByIdentHash (const i2p::data::IdentHash& ident) const;
 
 		private:
 
@@ -224,7 +228,7 @@ namespace client
 
 			void Accept ();
 
-			void HandleAccept(const boost::system::error_code& ecode, std::shared_ptr<I2CPSession::proto::socket> socket);
+			void HandleAccept(const boost::system::error_code& ecode, std::shared_ptr<boost::asio::ip::tcp::socket> socket);
 
 		private:
 
@@ -232,7 +236,7 @@ namespace client
 			I2CPMessageHandler m_MessagesHandlers[256];
 			std::map<uint16_t, std::shared_ptr<I2CPSession> > m_Sessions;
 
-			I2CPSession::proto::acceptor m_Acceptor;
+			boost::asio::ip::tcp::acceptor m_Acceptor;
 
 		public:
 
