@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2013-2020, The PurpleI2P Project
+* Copyright (c) 2013-2021, The PurpleI2P Project
 *
 * This file is part of Purple i2pd project and licensed under BSD3
 *
@@ -49,7 +49,7 @@ namespace client
 			void I2PConnect (const uint8_t * msg = nullptr, size_t len = 0);
 			void Connect (bool isUniqueLocal = true);
 			void Connect (const boost::asio::ip::address& localAddress);
-		
+
 		protected:
 
 			void Terminate ();
@@ -59,7 +59,7 @@ namespace client
 			virtual void Write (const uint8_t * buf, size_t len); // can be overloaded
 			void HandleWrite (const boost::system::error_code& ecode);
 			virtual void WriteToStream (const uint8_t * buf, size_t len); // can be overloaded
-		
+
 			void StreamReceive ();
 			void HandleStreamReceive (const boost::system::error_code& ecode, std::size_t bytes_transferred);
 			void HandleConnect (const boost::system::error_code& ecode);
@@ -105,7 +105,7 @@ namespace client
 		protected:
 
 			void Write (const uint8_t * buf, size_t len);
-			void WriteToStream (const uint8_t * buf, size_t len); 
+			void WriteToStream (const uint8_t * buf, size_t len);
 
 		private:
 
@@ -153,23 +153,29 @@ namespace client
 			void Stop ();
 
 			const char* GetName() { return m_Name.c_str (); }
+			void SetKeepAliveInterval (uint32_t keepAliveInterval);
 
 		private:
 
 			std::shared_ptr<const Address> GetAddress ();
 
+			void ScheduleKeepAliveTimer ();
+			void HandleKeepAliveTimer (const boost::system::error_code& ecode);
+
 		private:
 
 			std::string m_Name, m_Destination;
 			std::shared_ptr<const Address> m_Address;
 			int m_DestinationPort;
+			uint32_t m_KeepAliveInterval;
+			std::unique_ptr<boost::asio::deadline_timer> m_KeepAliveTimer;
 	};
 
 
 	/** 2 minute timeout for udp sessions */
 	const uint64_t I2P_UDP_SESSION_TIMEOUT = 1000 * 60 * 2;
-	const uint64_t I2P_UDP_REPLIABLE_DATAGRAM_INTERVAL = 100; // in milliseconds	
-	
+	const uint64_t I2P_UDP_REPLIABLE_DATAGRAM_INTERVAL = 100; // in milliseconds
+
 	/** max size for i2p udp */
 	const size_t I2P_UDP_MAX_MTU = 64*1024;
 
@@ -254,6 +260,10 @@ namespace client
 			std::vector<UDPSessionPtr> m_Sessions;
 			std::shared_ptr<i2p::client::ClientDestination> m_LocalDest;
 			UDPSessionPtr m_LastSession;
+
+		public:
+
+			bool isUpdated; // transient, used during reload only
 	};
 
 	class I2PUDPClientTunnel
@@ -283,7 +293,7 @@ namespace client
 			void TryResolving();
 
 		private:
-			
+
 			const std::string m_Name;
 			std::mutex m_SessionsMutex;
 			std::unordered_map<uint16_t, std::shared_ptr<UDPConvo> > m_Sessions; // maps i2p port -> local udp convo
@@ -298,6 +308,10 @@ namespace client
 			uint16_t RemotePort, m_LastPort;
 			bool m_cancel_resolve;
 			std::shared_ptr<UDPConvo> m_LastSession;
+
+		public:
+
+			bool isUpdated; // transient, used during reload only
 	};
 
 	class I2PServerTunnel: public I2PService
@@ -316,7 +330,7 @@ namespace client
 			bool IsUniqueLocal () const { return m_IsUniqueLocal; }
 
 			void SetLocalAddress (const std::string& localAddress);
-			
+
 			const std::string& GetAddress() const { return m_Address; }
 			int GetPort () const { return m_Port; };
 			uint16_t GetLocalPort () const { return m_PortDestination->GetLocalPort (); };
