@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2013-2020, The PurpleI2P Project
+* Copyright (c) 2013-2021, The PurpleI2P Project
 *
 * This file is part of Purple i2pd project and licensed under BSD3
 *
@@ -51,12 +51,13 @@ namespace util
 			MemoryPool (): m_Head (nullptr) {}
 			~MemoryPool ()
 			{
-				while (m_Head)
-				{
-					auto tmp = m_Head;
-					m_Head = static_cast<T*>(*(void * *)m_Head); // next
-					::operator delete ((void *)tmp);
-				}
+				CleanUp ();
+			}
+
+			void CleanUp ()
+			{
+				CleanUp (m_Head);
+				m_Head = nullptr;
 			}
 
 			template<typename... TArgs>
@@ -93,6 +94,18 @@ namespace util
 					std::bind (&MemoryPool<T>::Release, this, std::placeholders::_1));
 			}
 
+		protected:
+
+			void CleanUp (T * head)
+			{
+				while (head)
+				{
+					auto tmp = head;
+					head = static_cast<T*>(*(void * *)head); // next
+					::operator delete ((void *)tmp);
+				}
+			}
+
 		protected:
 
 			T * m_Head;
@@ -126,6 +139,24 @@ namespace util
 					this->Release (it);
 			}
 
+			template<typename... TArgs>
+			std::shared_ptr<T> AcquireSharedMt (TArgs&&... args)
+			{
+				return std::shared_ptr<T>(AcquireMt (std::forward<TArgs>(args)...),
+					std::bind<void (MemoryPoolMt<T>::*)(T *)> (&MemoryPoolMt<T>::ReleaseMt, this, std::placeholders::_1));
+			}
+
+			void CleanUpMt ()
+			{
+				T * head;
+				{
+					std::lock_guard<std::mutex> l(m_Mutex);
+					head = this->m_Head;
+					this->m_Head = nullptr;
+				}
+				if (head) this->CleanUp (head);
+			}
+
 		private:
 
 			std::mutex m_Mutex;
@@ -177,13 +208,13 @@ namespace util
 
 			SaveStateHelper (T& orig): m_Original (orig), m_Copy (orig) {};
 			~SaveStateHelper () { m_Original = m_Copy; };
-			
+
 		private:
 
 			T& m_Original;
 			T m_Copy;
-	};	
-	
+	};
+
 	namespace net
 	{
 		int GetMTU (const boost::asio::ip::address& localAddress);
